@@ -1,5 +1,5 @@
 package HTML::Template::Compiled::Utils;
-# $Id: Utils.pm,v 1.1 2005/09/01 23:32:28 tina Exp $
+# $Id: Utils.pm,v 1.2 2005/09/19 22:43:02 tinita Exp $
 $VERSION = "0.01";
 use strict;
 use warnings;
@@ -13,3 +13,13 @@ use constant PATH_METHOD => 1;
 use constant PATH_DEREF => 2;
 
 1;
+__END__
+
+=pod
+
+=head1 NAME
+
+HTML::Template::Compiled::Utils - Utility functions for HTML::Template::Compiled
+
+=cut
+
